﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Petzold.Epub
{
    public partial class EpubBook
    {
        IPackageAccessor packageAccessor;
        BookInformation bookInformation;
        TableOfContents tableOfContents;
        IList<HtmlFileReference> htmlFileList;
        string navigationFilePath;

        public EpubBook(IPackageAccessor packageAccessor, string packageFilePath)
        {
            this.PackageFilePath = packageFilePath;
            this.packageAccessor = packageAccessor;

            // Find directory preface for packageAccessor
            int index = packageFilePath.LastIndexOf('/');

            if (index != -1)
            {
                packageAccessor.DirectoryPreface = packageFilePath.Substring(0, index + 1);
                System.Diagnostics.Debug.WriteLine("EpubBook constructor setting DirectoryPreface to " + packageAccessor.DirectoryPreface);
            }
        }

        public string PackageFilePath { protected set; get; }

        public BookInformation BookInformation 
        {
            get
            {
                if (bookInformation == null)
                {
                    try
                    {
                        ParseOpfFile();
                    }
                    catch (Exception exc)
                    {
                        System.Diagnostics.Debug.WriteLine("From BookInformation: " + exc);
                    }
                }
                return bookInformation;
            }
        }

        public TableOfContents TableOfContents 
        {
            get
            {
                if (tableOfContents == null)
                {
                    if (String.IsNullOrEmpty(navigationFilePath))
                    {
                        try
                        {
                            ParseOpfFile();
                        }
                        catch (Exception exc)
                        {
                            System.Diagnostics.Debug.WriteLine("From TableOfContents: " + exc);
                        }
                    }

                    try
                    {
                        ParseNcxFile();
                    }
                    catch (Exception exc)
                    {
                        System.Diagnostics.Debug.WriteLine("From TableOfContents: " + exc);
                    }
                }
                return tableOfContents;
            }
        }

        public IList<HtmlFileReference> HtmlFileList
        {
            get
            {
                if (htmlFileList == null)
                    ParseOpfFile();
                    
                return htmlFileList;
            }
        }

        // TODO: PageSize -- need platform-independent Size structure!

        public double Zoom { set; get; }


        // TODO:  These probably should be fileIndex rather than chapterNumber

        public object GetPage(int chapterNumber, int pageNumber)
        {
            return null;
        }

        public object GetLastPage(int chapterNumber, out int pageNumber)
        {
            pageNumber = 0;
            return null;
        }

        public object QueryLastPage(int chapterNumber, out int pageNumber)
        {
            pageNumber = 0;
            return null;
        }

        public object GetPage(double fraction, out int chapterNumber, out int pageNumber)
        {
            chapterNumber = 0;
            pageNumber = 0;
            return null;
        }

        // TODO:  Plus, there should be one that goes to the first page of a chapter
    }
}
