﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Petzold.Epub;

namespace TheNewEpublic
{
    public sealed partial class BookReaderPage
    {
        BookReference bookReference;
        EpubBook epubBook;

        public BookReaderPage(BookReference bookReference)
        {
            InitializeComponent();
            this.bookReference = bookReference;
            Loaded += OnLoaded;
        }

        async void OnLoaded(object sender, RoutedEventArgs args)
        {
            System.Diagnostics.Debug.WriteLine("attempted to open " + bookReference.FileName);

 	        StorageFile storageFile = await StorageFile.GetFileFromPathAsync(bookReference.FileName);

            System.Diagnostics.Debug.WriteLine("file size = " + storageFile.Size);

            byte[] buffer = await Helpers.ReadStorageFileAsync(storageFile);

            using (MemoryStream memoryStream = new MemoryStream(buffer, false))
            {
                ZipPackageAccessor packageAccessor = new ZipPackageAccessor(memoryStream);
                epubBook = new EpubBook(packageAccessor, bookReference.OpfFileName);
                PageTitle.Text = epubBook.BookInformation.Titles[0];
                contentsListBox.ItemsSource = epubBook.TableOfContents.Chapters;
            }
        }

        void OnBackButtonClick(object sender, RoutedEventArgs args)
        {
            Window.Current.Content = new MainPage();
        }

        #region Automatic Stuff for Blank Page

        // View state management for switching among Full, Fill, Snapped, and Portrait states

        private DisplayPropertiesEventHandler _displayHandler;
        private TypedEventHandler<ApplicationLayout, ApplicationLayoutChangedEventArgs> _layoutHandler;

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            if (_displayHandler == null)
            {
                _displayHandler = Page_OrientationChanged;
                _layoutHandler = Page_LayoutChanged;
            }
            DisplayProperties.OrientationChanged += _displayHandler;
            ApplicationLayout.GetForCurrentView().LayoutChanged += _layoutHandler;
            SetCurrentOrientation(this);
        }

        private void Page_Unloaded(object sender, RoutedEventArgs e)
        {
            DisplayProperties.OrientationChanged -= _displayHandler;
            ApplicationLayout.GetForCurrentView().LayoutChanged -= _layoutHandler;
        }

        private void Page_LayoutChanged(object sender, ApplicationLayoutChangedEventArgs e)
        {
            SetCurrentOrientation(this);
        }

        private void Page_OrientationChanged(object sender)
        {
            SetCurrentOrientation(this);
        }

        private void SetCurrentOrientation(Control viewStateAwareControl)
        {
            VisualStateManager.GoToState(viewStateAwareControl, this.GetViewState(), false);
        }

        private String GetViewState()
        {
            var orientation = DisplayProperties.CurrentOrientation;
            if (orientation == DisplayOrientations.Portrait ||
                orientation == DisplayOrientations.PortraitFlipped) return "Portrait";
            var layout = ApplicationLayout.Value;
            if (layout == ApplicationLayoutState.Filled) return "Fill";
            if (layout == ApplicationLayoutState.Snapped) return "Snapped";
            return "Full";
        }

        #endregion


    }
}
